package com.bwie.auth.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bwie.auth.feign.UserFeign;
import com.bwie.auth.service.AuthService;
import com.bwie.common.constants.JwtConstants;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.User;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

/**
 * @Author：杨鹏
 * @Package：com.bwie.auth.service.impl
 * @Project：yp_yuekao812
 * @name：AuthServiceImpl
 * @Date：2024/8/12 9:53
 */
@Service
public class AuthServiceImpl implements AuthService {

    @Autowired
    private UserFeign userFeign;

    @Autowired
    private HttpServletRequest request;

    @Autowired
    private RabbitTemplate rabbitTemplate;

    @Autowired
    private StringRedisTemplate redisTemplate;

    /**
     * 登录
     */
    @Override
    public Result<User> login(User user) {
        if (StringUtils.isAllEmpty(user.getUserName(),user.getUserPwd())){
            return Result.error("用户名称或密码不能为空");
        }
        Result<User> login = userFeign.login(user);
        User loginData = login.getData();
        if (loginData == null){
            return Result.error("该用户名不存在,请重新输入或者注册");
        }
        if (!user.getUserPwd().equals(loginData.getUserPwd())){
            return Result.error("密码错误");
        }
        String userKey = UUID.randomUUID().toString().replace("-", "");
        Map<String, Object> map = new HashMap<>();
        map.put(JwtConstants.USER_KEY,userKey);
        String token = JwtUtils.createToken(map);
        loginData.setToken(token);
        redisTemplate.opsForValue().set(TokenConstants.LOGIN_TOKEN_KEY+userKey, JSONObject.toJSONString(loginData),
                TokenConstants.EXPIRATION, TimeUnit.MINUTES);
        //	用户每次登陆系统需要给用户发送短息（3分）
        //	用户短信统一先放入MQ当中（2分）
        rabbitTemplate.convertAndSend("http_yp_lv",loginData.getUserPhone(),message -> {
            message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
            return message;
        });
        return Result.success(loginData);
    }

    /**
     * 登录认证
     */
    @Override
    public Result info() {
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String user = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        return Result.success(JSONObject.parseObject(user,User.class));
    }

    /**
     * 退出登录
     */
    @Override
    public void logout() {
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        redisTemplate.delete(TokenConstants.LOGIN_TOKEN_KEY + userKey);
    }

    /**
     * 注册
     */
    @Override
    public Result addUser(User user) {

        if (StringUtils.isAllEmpty(user.getUserPhone())){
            return Result.error("用户手机号不能为空");
        }
        Result<User> listUserPhone = userFeign.listUserPhone(user);
        User listUserPhoneData = listUserPhone.getData();
        if (listUserPhoneData != null){
            return Result.error("该用户手机号已注册,请重新输入");
        }

        if (StringUtils.isAllEmpty(user.getUserName(),user.getUserPwd())){
            return Result.error("用户名称或密码不能为空");
        }

        Result<User> login = userFeign.login(user);
        User loginData = login.getData();
        if (loginData != null){
            return Result.error("该用户名已经存在,请重新输入");
        }

        Result<Integer> integerResult = userFeign.addUser(user);
        Integer integer = integerResult.getData();
        if (integer < 0){
            return Result.error("注册失败");
        }
        //	用户使用手机号注册，注册成功后给用户发送短信（3分）
        //	用户短信统一先放入MQ当中（2分）
        rabbitTemplate.convertAndSend("http_yp_lv",user.getUserPhone(),message -> {
            message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
            return message;
        });
        return Result.success("注册成功");
    }
}
